@@ -22,8 +22,7 @@
 #
 # $Id$
 #
-"""Read from and write to 7zip format archives.
-"""
+"""Read 7zip format archives."""
 
 from array import array
 from binascii import unhexlify
@@ -131,6 +130,8 @@ PROPERTY_LAST_WRITE_TIME         = unhexlify('14')  # '\x14'
 PROPERTY_ATTRIBUTES              = unhexlify('15')  # '\x15'
 PROPERTY_COMMENT                 = unhexlify('16')  # '\x16'
 PROPERTY_ENCODED_HEADER          = unhexlify('17')  # '\x17'
+PROPERTY_START_POS               = unhexlify('18')  # '\x18'
+PROPERTY_DUMMY                   = unhexlify('19')  # '\x19'
 
 COMPRESSION_METHOD_COPY          = unhexlify('00')  # '\x00'
 COMPRESSION_METHOD_LZMA          = unhexlify('03')  # '\x03'
@@ -483,6 +484,11 @@ class FilesInfo(Base):
                 break
                 
             size = self._read64Bit(file)
+            if typ == PROPERTY_DUMMY:
+                # Added by newer versions of 7z to adjust padding.
+                file.seek(size, os.SEEK_CUR)
+                continue
+
             buffer = BytesIO(file.read(size))
             if typ == PROPERTY_EMPTY_STREAM:
                 isempty = self._readBoolean(buffer, self.numfiles)
@@ -788,8 +794,9 @@ class Archive7z(Base):
                     '_uncompressed': uncompressed,
                 }
                 tmp = ArchiveFile(info, 0, src_start, folder, self)
-                folderdata = tmp.read()
-                src_start += uncompressed[-1]
+                uncompressed_size = uncompressed[-1]
+                folderdata = tmp.read()[:uncompressed_size]
+                src_start += uncompressed_size
 
                 if folder.digestdefined:
                     if not self.checkcrc(folder.crc, folderdata):
@@ -810,15 +817,19 @@ class Archive7z(Base):
         
         self.header = Header(buffer)
         files = self.header.files
-        folders = self.header.main_streams.unpackinfo.folders
-        packinfo = self.header.main_streams.packinfo
-        subinfo = self.header.main_streams.substreamsinfo
-        packsizes = packinfo.packsizes
-        self.solid = packinfo.numstreams == 1
-        if hasattr(subinfo, 'unpacksizes'):
-            unpacksizes = subinfo.unpacksizes
+        if hasattr(self.header, 'main_streams'):
+            folders = self.header.main_streams.unpackinfo.folders
+            packinfo = self.header.main_streams.packinfo
+            subinfo = self.header.main_streams.substreamsinfo
+            packsizes = packinfo.packsizes
+            self.solid = packinfo.numstreams == 1
+            if hasattr(subinfo, 'unpacksizes'):
+                unpacksizes = subinfo.unpacksizes
+            else:
+                unpacksizes = [x.unpacksizes for x in folders]
         else:
-            unpacksizes = [x.unpacksizes for x in folders]
+            # TODO(fancycode): is it necessary to provide empty values for folder, packinfo, etc?
+            self.solid = False
         
         fidx = 0
         obidx = 0
